@@ -3,6 +3,8 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
+ * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -36,6 +38,42 @@
     Change History (most recent first):
 
 $Log: Identify.c,v $
+Revision 1.23  2004/05/18 23:51:26  cheshire
+Tidy up all checkin comments to use consistent "<rdar://problem/xxxxxxx>" format for bug numbers
+
+Revision 1.22  2004/04/20 22:43:28  cheshire
+Use _services._dns-sd._udp query, as documented in
+<http://files.dns-sd.org/draft-cheshire-dnsext-dns-sd-02.txt>
+
+Revision 1.21  2004/01/28 21:38:57  cheshire
+Also ask target host for _services._mdns._udp.local. list
+
+Revision 1.20  2004/01/28 19:04:38  cheshire
+Fix Ctrl-C handling when multiple targets are specified
+
+Revision 1.19  2004/01/28 03:49:30  cheshire
+Enhanced mDNSIdentify to make use of new targeted-query capability
+
+Revision 1.18  2004/01/27 19:06:51  cheshire
+Remove workaround for WWDC 2003 bug; no one has run that buggy build for a long time
+
+Revision 1.17  2004/01/22 03:57:00  cheshire
+Use the new meta-interface mDNSInterface_ForceMCast. This restores mDNSIdentify's
+ability to use multicast queries with non-link-local target addresses, like 17.x.x.x.
+
+Revision 1.16  2004/01/22 00:03:32  cheshire
+Add while() loop so that a list of targets may be specified on the command line
+
+Revision 1.15  2004/01/21 21:55:06  cheshire
+Don't need to wait for timeout once we've got the information we wanted
+
+Revision 1.14  2003/12/17 00:51:22  cheshire
+Changed mDNSNetMonitor and mDNSIdentify to link the object files
+instead of #including the "DNSCommon.c" "uDNS.c" and source files
+
+Revision 1.13  2003/12/13 03:05:28  ksekar
+<rdar://problem/3192548>: DynDNS: Unicast query of service records
+
 Revision 1.12  2003/11/14 21:27:09  cheshire
 <rdar://problem/3484766>: Security: Crashing bug in mDNSResponder
 Fix code that should use buffer size MAX_ESCAPED_DOMAIN_NAME (1005) instead of 256-byte buffers.
@@ -114,6 +152,7 @@ static CacheRecord gRRCache[RR_CACHE_SIZE];
 static volatile int StopNow;	// 0 means running, 1 means stop because we got an answer, 2 means stop because of Ctrl-C
 static volatile int NumAnswers, NumAddr, NumAAAA, NumHINFO;
 static char hostname[MAX_ESCAPED_DOMAIN_NAME], hardware[256], software[256];
+static mDNSAddr lastsrc, hostaddr, target;
 static mDNSOpaque16 lastid, id;
 
 //*************************************************************************************************************
@@ -142,6 +181,11 @@ mDNSexport void mDNSCoreReceive(mDNS *const m, DNSMessage *const msg, const mDNS
 	{
 	// Snag copy of header ID, then call through
 	lastid = msg->h.id;
+	lastsrc = *srcaddr;
+
+	// We *want* to allow off-net unicast responses here.
+	// For now, the simplest way to allow that is to smash the TTL to 255 so that mDNSCore doesn't reject the packet
+	ttl = 255;
 	__MDNS__mDNSCoreReceive(m, msg, end, srcaddr, srcport, dstaddr, dstport, InterfaceID, ttl);
 	}
 
@@ -151,9 +195,12 @@ static void NameCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 	(void)question;	// Unused
 	(void)AddRecord;// Unused
 	if (!id.NotAnInteger) id = lastid;
-	ConvertDomainNameToCString(&answer->rdata->u.name, hostname);
-	StopNow = 1;
-	mprintf("%##s %s %##s\n", answer->name.c, DNSTypeName(answer->rrtype), &answer->rdata->u.name.c);
+	if (answer->rrtype == kDNSType_PTR || answer->rrtype == kDNSType_CNAME)
+		{
+		ConvertDomainNameToCString(&answer->rdata->u.name, hostname);
+		StopNow = 1;
+		mprintf("%##s %s %##s\n", answer->name.c, DNSTypeName(answer->rrtype), &answer->rdata->u.name.c);
+		}
 	}
 
 static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, mDNSBool AddRecord)
@@ -167,6 +214,8 @@ static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 		NumAnswers++;
 		NumAddr++;
 		mprintf("%##s %s %.4a\n", answer->name.c, DNSTypeName(answer->rrtype), &answer->rdata->u.ip);
+		hostaddr.type = mDNSAddrType_IPv4;	// Prefer v4 target to v6 target, for now
+		hostaddr.ip.v4 = answer->rdata->u.ip;
 		}
 	else if (answer->rrtype == kDNSType_AAAA)
 		{
@@ -174,6 +223,11 @@ static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 		NumAnswers++;
 		NumAAAA++;
 		mprintf("%##s %s %.16a\n", answer->name.c, DNSTypeName(answer->rrtype), &answer->rdata->u.ipv6);
+		if (!hostaddr.type)	// Prefer v4 target to v6 target, for now
+			{
+			hostaddr.type = mDNSAddrType_IPv6;
+			hostaddr.ip.v6 = answer->rdata->u.ipv6;
+			}
 		}
 	else if (answer->rrtype == kDNSType_HINFO)
 		{
@@ -186,6 +240,26 @@ static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 		NumAnswers++;
 		NumHINFO++;
 		}
+
+	// If we've got everything we're looking for, don't need to wait any more
+	if (NumHINFO && (NumAddr || NumAAAA)) StopNow = 1;
+	}
+
+static void ServicesCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, mDNSBool AddRecord)
+	{
+	(void)m;		// Unused
+	(void)question;	// Unused
+	(void)AddRecord;// Unused
+	// Right now the mDNSCore targeted-query code is incomplete --
+	// it issues targeted queries, but accepts answers from anywhere
+	// For now, we'll just filter responses here so we don't get confused by responses from someone else
+	if (answer->rrtype == kDNSType_PTR && mDNSSameAddress(&lastsrc, &target))
+		{
+		NumAnswers++;
+		NumAddr++;
+		mprintf("%##s %s %##s\n", answer->name.c, DNSTypeName(answer->rrtype), &answer->rdata->u.name.c);
+		StopNow = 1;
+		}
 	}
 
 mDNSexport void WaitForAnswer(mDNS *const m, int seconds)
@@ -215,11 +289,13 @@ mDNSexport void WaitForAnswer(mDNS *const m, int seconds)
 		}
 	}
 
-mDNSlocal mStatus StartQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, mDNSQuestionCallback callback)
+mDNSlocal mStatus StartQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNSAddr *target, mDNSQuestionCallback callback)
 	{
 	if (qname) MakeDomainNameFromDNSNameString(&q->qname, qname);
-
-	q->InterfaceID      = mDNSInterface_Any;
+	q->Target           = target ? *target : zeroAddr;
+	q->TargetPort       = MulticastDNSPort;
+	q->TargetQID        = zeroID;
+	q->InterfaceID      = mDNSInterface_ForceMCast;
 	q->qtype            = qtype;
 	q->qclass           = kDNSClass_IN;
 	q->QuestionCallback = callback;
@@ -229,18 +305,28 @@ mDNSlocal mStatus StartQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, mDNSQue
 	return(mDNS_StartQuery(&mDNSStorage, q));
 	}
 
-mDNSlocal int DoQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, mDNSQuestionCallback callback)
+mDNSlocal void DoOneQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNSAddr *target, mDNSQuestionCallback callback)
 	{
-	mStatus status = StartQuery(q, qname, qtype, callback);
+	mStatus status = StartQuery(q, qname, qtype, target, callback);
 	if (status != mStatus_NoError)
 		StopNow = 2;
 	else
 		{
 		WaitForAnswer(&mDNSStorage, 4);
 		mDNS_StopQuery(&mDNSStorage, q);
-		if (StopNow == 0 && NumAnswers == 0)
-			printf("%s %s *** No Answer ***\n", qname, DNSTypeName(qtype));
 		}
+	}
+
+mDNSlocal int DoQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNSAddr *target, mDNSQuestionCallback callback)
+	{
+	DoOneQuery(q, qname, qtype, target, callback);
+	if (StopNow == 0 && target && target->type)
+		{
+		mprintf("%##s %s Trying multicast\n", q->qname.c, DNSTypeName(q->qtype));
+		DoOneQuery(q, qname, qtype, NULL, callback);
+		}
+	if (StopNow == 0 && NumAnswers == 0)
+		mprintf("%##s %s *** No Answer ***\n", q->qname.c, DNSTypeName(q->qtype));
 	return(StopNow);
 	}
 
@@ -254,6 +340,7 @@ mDNSlocal void HandleSIG(int signal)
 
 mDNSexport int main(int argc, char **argv)
 	{
+	int this_arg = 1;
 	mStatus status;
 	struct in_addr s4;
 	struct in6_addr s6;
@@ -262,6 +349,9 @@ mDNSexport int main(int argc, char **argv)
 
 	if (argc < 2) goto usage;
 	
+	// Since this is a special command-line tool, we want LogMsg() errors to go to stderr, not syslog
+	mDNS_DebugMode = mDNStrue;
+	
     // Initialise the mDNS core.
 	status = mDNS_Init(&mDNSStorage, &PlatformStorage,
     	gRRCache, RR_CACHE_SIZE,
@@ -272,63 +362,80 @@ mDNSexport int main(int argc, char **argv)
 	signal(SIGINT, HandleSIG);	// SIGINT is what you get for a Ctrl-C
 	signal(SIGTERM, HandleSIG);
 
-	if (inet_pton(AF_INET, argv[1], &s4) == 1)
+	while (this_arg < argc)
 		{
-		mDNSu8 *p = (mDNSu8 *)&s4;
-		mDNS_snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d.in-addr.arpa.", p[3], p[2], p[1], p[0]);
-		printf("%s\n", buffer);
-		if (DoQuery(&q, buffer, kDNSType_PTR, NameCallback) != 1) goto exit;
-		}
-	else if (inet_pton(AF_INET6, argv[1], &s6) == 1)
-		{
-		DNSQuestion q1, q2;
-		int i;
-		mDNSu8 *p = (mDNSu8 *)&s6;
-		for (i = 0; i < 16; i++)
+		char *arg = argv[this_arg++];
+		if (this_arg > 2) printf("\n");
+
+		lastid = id = zeroID;
+		hostaddr = target = zeroAddr;
+		hostname[0] = hardware[0] = software[0] = 0;
+		NumAddr = NumAAAA = NumHINFO = 0;
+
+		if (inet_pton(AF_INET, arg, &s4) == 1)
 			{
-			static const char hexValues[] = "0123456789ABCDEF";
-			buffer[i * 4    ] = hexValues[p[15-i] & 0x0F];
-			buffer[i * 4 + 1] = '.';
-			buffer[i * 4 + 2] = hexValues[p[15-i] >> 4];
-			buffer[i * 4 + 3] = '.';
+			mDNSu8 *p = (mDNSu8 *)&s4;
+			mDNS_snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d.in-addr.arpa.", p[3], p[2], p[1], p[0]);
+			printf("%s\n", buffer);
+			target.type = mDNSAddrType_IPv4;
+			target.ip.v4.NotAnInteger = s4.s_addr;
+			DoQuery(&q, buffer, kDNSType_PTR, &target, NameCallback);
+			if (StopNow == 2) break;
 			}
-		mDNS_snprintf(&buffer[64], sizeof(buffer)-64, "ip6.arpa.");
-		MakeDomainNameFromDNSNameString(&q1.qname, buffer);
-		mDNS_snprintf(&buffer[32], sizeof(buffer)-32, "ip6.arpa.");	// Workaround for WWDC bug
-		MakeDomainNameFromDNSNameString(&q2.qname, buffer);
-		StartQuery(&q1, NULL, kDNSType_PTR, NameCallback);
-		StartQuery(&q2, NULL, kDNSType_PTR, NameCallback);
-		WaitForAnswer(&mDNSStorage, 4);
-		mDNS_StopQuery(&mDNSStorage, &q1);
-		mDNS_StopQuery(&mDNSStorage, &q2);
-		if (StopNow != 1) { mprintf("%##s %s *** No Answer ***\n", q1.qname.c, DNSTypeName(q1.qtype)); goto exit; }
+		else if (inet_pton(AF_INET6, arg, &s6) == 1)
+			{
+			int i;
+			mDNSu8 *p = (mDNSu8 *)&s6;
+			for (i = 0; i < 16; i++)
+				{
+				static const char hexValues[] = "0123456789ABCDEF";
+				buffer[i * 4    ] = hexValues[p[15-i] & 0x0F];
+				buffer[i * 4 + 1] = '.';
+				buffer[i * 4 + 2] = hexValues[p[15-i] >> 4];
+				buffer[i * 4 + 3] = '.';
+				}
+			mDNS_snprintf(&buffer[64], sizeof(buffer)-64, "ip6.arpa.");
+			target.type = mDNSAddrType_IPv6;
+			bcopy(&s6, &target.ip.v6, sizeof(target.ip.v6));
+			DoQuery(&q, buffer, kDNSType_PTR, &target, NameCallback);
+			if (StopNow == 2) break;
+			}
+		else
+			strcpy(hostname, arg);
+	
+		// Now we have the host name; get its A, AAAA, and HINFO
+		if (hostname[0]) DoQuery(&q, hostname, kDNSQType_ANY, &target, InfoCallback);
+		if (StopNow == 2) break;
+	
+		if (hardware[0] || software[0])
+			{
+			DNSQuestion q1, q2;
+			printf("HINFO Hardware: %s\n", hardware);
+			printf("HINFO Software: %s\n", software);
+			// We need to make sure the services query is targeted
+			if (target.type == 0) target = hostaddr;
+			StartQuery(&q1, "_services._mdns._udp.local.",   kDNSQType_ANY, &target, ServicesCallback);
+			StartQuery(&q2, "_services._dns-sd._udp.local.", kDNSQType_ANY, &target, ServicesCallback);
+			WaitForAnswer(&mDNSStorage, 4);
+			mDNS_StopQuery(&mDNSStorage, &q1);
+			mDNS_StopQuery(&mDNSStorage, &q2);
+			if (StopNow == 2) break;
+			}
+		else if (NumAnswers)
+			{
+			printf("Host has no HINFO record; Best guess is ");
+			if (id.b[1]) printf("mDNSResponder-%d\n", id.b[1]);
+			else if (NumAAAA) printf("very early Panther build (mDNSResponder-33 or earlier)\n");
+			else printf("Jaguar version of mDNSResponder with no IPv6 support\n");
+			}
+		else
+			printf("Incorrect dot-local hostname, address, or no mDNSResponder running on that machine\n");
 		}
-	else
-		strcpy(hostname, argv[1]);
 
-	// Now we have the host name; get its A, AAAA, and HINFO
-	if (DoQuery(&q, hostname, kDNSQType_ANY, InfoCallback) == 2) goto exit;	// Interrupted with Ctrl-C
-
-	if (hardware[0] || software[0])
-		{
-		printf("HINFO Hardware: %s\n", hardware);
-		printf("HINFO Software: %s\n", software);
-		}
-	else if (NumAnswers)
-		{
-		printf("Host has no HINFO record; Best guess is ");
-		if (id.b[1]) printf("mDNSResponder-%d\n", id.b[1]);
-		else if (NumAAAA) printf("very early Panther build (mDNSResponder-33 or earlier)\n");
-		else printf("Jaguar version of mDNSResponder with no IPv6 support\n");
-		}
-	else
-		printf("Incorrect dot-local hostname, address, or no mDNSResponder running on that machine\n");
-
-exit:
 	mDNS_Close(&mDNSStorage);
 	return(0);
 
 usage:
-	fprintf(stderr, "%s <dot-local hostname> or <IPv4 address> or <IPv6 address>\n", argv[0]);
+	fprintf(stderr, "%s <dot-local hostname> or <IPv4 address> or <IPv6 address> ...\n", argv[0]);
 	return(-1);
 	}
